/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/


/*********************************************************
________________________
Definition of constants:
#define NUM_OF_NODES			10
#define MAX_NUM_OF_MULTIPATHS 	10
______________________
Definition of structs:

// Routing table
struct routing_table_entry{
	//uint8_t destination_node;	// ID of destination node
	uint8_t next_hop;				// NodeID of the next node
	float pheromone_value;			// Value of the pheromone for the route
	uint8_t flag;					// Additional flag to store some data
};
//Create a 2D table - first row for nodes, second row for multipaths
static struct routing_table_entry routing_table[NUM_OF_NODES][MAX_NUM_OF_MULTIPATHS];

// Data of a BANT or FANT packet
struct data_of_fant_bant{
	uint8_t source_node;			// ID of the source node (sending the FANT)
	uint8_t destination_node;	// ID of the destination node
	uint16_t sequence_number;		// Sequence number: Number of FANTs or BANTs of the source node (incremental)
	uint16_t hop_counter;			// Number of hops of the packet
	uint8_t pre_predecessor;		// Node ID of the pre-predecessor
	uint8_t flag;					// Additional flag to store data (e.g. if BANT or FANT)
};
struct data_of_fant_bant data_of_actual_bant;
struct data_of_fant_bant data_of_actual_fant;

______________________
Definition of methods:

// Send a FANT every 5 seconds to node id; 
// Timeout after 60 seconds
// @param
// uint8_t id: ID of the destination node 
// return uint8_t: Success/Error-Code
uint8_t Find_Route (uint8_t id);


// Check if a route to node id exists in the routing tables
// @param
// uint8_t id: ID of the destination node
// return uint8_t:  Success/Error-Code
uint8_t Check_Existing_Routes (uint8_t id);


// Send (broadcast) a FANT packet to node id
// @param
// uint8_t id: IF of the destination node
// struct data *data: Pointer to the values/data for the FANT
// uint8_t sizeofdata: Size of the data to send
// return uint8_t: Success/Error-Code
uint8_t Send_FANT (uint8_t id, struct data *data, uint8_t datasize);


// Send (broadcast) a BANT packet to node id
// @param
// uint8_t id: IF of the destination node
// struct data *data: Pointer to the values/data for the FANT
// uint8_t sizeofdata: Size of the data to send
// return uint8_t: Success/Error-Code
uint8_t Send_BANT (uint8_t id, struct data *data, uint8_t datasize);


// If node got a FANT - process it (called in Process_radioHandler)
// @param
// struct data_of_fant_bant *data_of_actual_bant: Value of the packet content
// return uint8_t: Success/Error-Code 
uint8_t Process_FANT (struct data_of_fant_bant *data_of_actual_fant);

// If node got a BANT - process it (called in Process_radioHandler)
// @param
// struct data_of_fant_bant *data_of_actual_bant: Value of the packet content
// return uint8_t: Success/Error-Code 
uint8_t Process_BANT (struct data_of_fant_bant *data_of_actual_bant);

*********************************************************/
#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <string.h>
#include "ScatterWeb.process.h"
#include "DataPacketHandler.h"



uint8_t size_of_data=0;


uint16_t i,j;
uint8_t counter = 0;
uint8_t user_data = 0;

//Method
void callMe();
void blinkBlank(void* loops);
void trafficGenerator();

uint8_t Send_FANT_BANT (struct data_of_fant_bant *p_fant_bant, uint8_t packet_type );
uint8_t Check_And_Add_FANT_table(struct data_of_fant_bant *p_fant);
uint8_t Check_And_Add_BANT_table(struct data_of_fant_bant *p_bant);
uint8_t Add_Routing_Table_Entry(uint8_t destination_node, uint8_t next_hop, uint8_t flag, uint16_t hops);

bool Process_radioHandler(struct netpacket_handler_args* args)
{

	/*
		Notes: * The received packet data becomes invalid when this handler is completed.
	*/

	// Pass packet through command interface
	if ( Cmd_radioHandler( args ) ) {
		// packet handled by messaging module
		return true;
	}

	if ( args->cpaResult == ADRCOMP_OTHER ) {
		// intercept packets for other nodes here
	} else if ( args->cpaResult == ADRCOMP_BROADCAST ) {
		// process broadcast packets here
		struct data_of_fant_bant *p_fant;
		p_fant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
		
		struct data_of_fant_bant *p_bant;
		p_bant = (struct data_of_fant_bant*)malloc(sizeof(struct data_of_fant_bant));
		
		switch ( args->netheader->type ) {
			
			case USERDEFINED_PACKET+1:
			//Process FANT packet
			/*Structure of FANT packet
			
			|###########|-------------|-----------------|------------------|------|-----------|-------------|
			| Netheader | Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter | 
			|###########|-------------|-----------------|------------------|------|-----------|-------------|			
							uint8_t		    uint8_t		      uint8_t		uint8_t  uint16_t	 uint16_t
			*/
				//uint8_t source_node;			
				//uint8_t destination_node;	
				//uint16_t sequence_number;		
				//uint16_t hop_counter;			
				//uint8_t pre_predecessor;		
				//uint8_t flag;		
				printf("Received FANT\r\n");
				//Get source node
				p_fant->source_node = *((uint8_t *) (args->payload));
				printf("source_node: %d ** ", p_fant->source_node);
				
				//Get pre_predecessor
				p_fant->pre_predecessor = *((uint8_t *) (args->payload) + 1);
				printf("pre_predecessor: %d ** ", p_fant->pre_predecessor);
				
				//Get Destination node
				p_fant->destination_node = *((uint8_t *) (args->payload) + 2);
				printf("destination_node: %d ** ", p_fant->destination_node);
				
				//Get Flag
				p_fant->flag = *((uint8_t *) (args->payload) + 3);
				printf("flag: %d ** ", p_fant->flag);
				
				//Get Sequence_Number
				p_fant->sequence_number = *((uint16_t *) (args->payload) + 2);
				printf("sequence_number: %d ** ", p_fant->sequence_number);
				
				//Get Hop_Count
				p_fant->hop_counter = *((uint16_t *) (args->payload) + 3);
				printf("hop_counter: %d\r\n", p_fant->hop_counter);
				
				//Loop
				//Send_FANT(p_fant);
				
				
				
				if (p_fant->destination_node == Configuration.id){
					//We are the destination of the FANT packet
					printf("I am the destination of this FANT");
					Add_Routing_Table_Entry(p_fant->source_node, args->netheader->from, p_fant->flag, p_fant->hop_counter);
					
				}				
				//Make an entry in FANT table fant_table[SIZE_OF_FANT_TABLE]
				//check if we have already seen this packet
				else if (Check_And_Add_FANT_table(p_fant) == PACKET_IN_FANT_TABLE){
					//Do not forward			
					printf("FANT not added to FT\r\n");
					//Check if we have send this packet to our neighbor and it sends it back to us
					if (p_fant->pre_predecessor == Configuration.id){
						//ignore
					}else{
						//Make entry in routing table
						//uint8_t destination_node, uint8_t next_hop, uint8_t flag
						Add_Routing_Table_Entry(p_fant->source_node, args->netheader->from, p_fant->flag, p_fant->hop_counter);
					}
				}
				else {//we havent seen this packet
					//Make entry in routing table
					printf("Added FANT to FT\r\n");
					Add_Routing_Table_Entry(p_fant->source_node, args->netheader->from, p_fant->flag, p_fant->hop_counter);
					
					//Change pre-predecessor of packet to actual predecessor
					p_fant->pre_predecessor = args->netheader->from;
					//increment hops
					p_fant->hop_counter++;
					//Forward FANT packet
					Send_FANT_BANT(p_fant, PACKET_TYPE_BANT);
				}
				
				free(p_fant);
				
				break;
			
			case USERDEFINED_PACKET+2:
			/*Structure of BANT packet
			
			|###########|-------------|-----------------|------------------|------|-----------|-------------|
			| Netheader | Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter | 
			|###########|-------------|-----------------|------------------|------|-----------|-------------|			
							uint8_t		    uint8_t		      uint8_t		uint8_t  uint16_t	 uint16_t
			*/
				//uint8_t source_node;			
				//uint8_t destination_node;	
				//uint16_t sequence_number;		
				//uint16_t hop_counter;			
				//uint8_t pre_predecessor;		
				//uint8_t flag;		
				printf("Received BANT\r\n");
				//Get source node
				p_bant->source_node = *((uint8_t *) (args->payload));
				printf("source_node: %d ** ", p_bant->source_node);
				
				//Get pre_predecessor
				p_bant->pre_predecessor = *((uint8_t *) (args->payload) + 1);
				printf("pre_predecessor: %d ** ", p_bant->pre_predecessor);
				
				//Get Destination node
				p_bant->destination_node = *((uint8_t *) (args->payload) + 2);
				printf("destination_node: %d ** ", p_bant->destination_node);
				
				//Get Flag
				p_bant->flag = *((uint8_t *) (args->payload) + 3);
				printf("flag: %d ** ", p_bant->flag);
				
				//Get Sequence_Number
				p_bant->sequence_number = *((uint16_t *) (args->payload) + 2);
				printf("sequence_number: %d ** ", p_bant->sequence_number);
				
				//Get Hop_Count
				p_bant->hop_counter = *((uint16_t *) (args->payload) + 3);
				printf("hop_counter: %d\r\n", p_bant->hop_counter);
				
				//Loop
				//Send_FANT(p_bant);
				
				
				
				if (p_bant->destination_node == Configuration.id){
					//We are the destination of the BANT packet
					printf("I am the destination of this BANT");
					Add_Routing_Table_Entry(p_bant->source_node, args->netheader->from, p_bant->flag, p_bant->hop_counter);
					
				}				
				//Make an entry in BANT table bant_table[SIZE_OF_BANT_TABLE]
				//check if we have already seen this packet
				else if (Check_And_Add_BANT_table(p_bant) == PACKET_IN_BANT_TABLE){
					//Do not forward			
					printf("BANT not added to BT\r\n");
					//Check if we have send this packet to our neighbor and it sends it back to us
					if (p_bant->pre_predecessor == Configuration.id){
						//ignore
					}else{
						//Make entry in routing table
						//uint8_t destination_node, uint8_t next_hop, uint8_t flag
						Add_Routing_Table_Entry(p_bant->source_node, args->netheader->from, p_bant->flag, p_bant->hop_counter);
					}
				}
				else {//we havent seen this packet
					//Make entry in routing table
					printf("Added BANT to BT\r\n");
					Add_Routing_Table_Entry(p_bant->source_node, args->netheader->from, p_bant->flag, p_bant->hop_counter);
					
					//Change pre-predecessor of packet to actual predecessor
					p_bant->pre_predecessor = args->netheader->from;
					//increment hops
					p_bant->hop_counter++;
					//Forward FANT packet
					Send_FANT_BANT(p_bant, PACKET_TYPE_BANT);
				}
				
				free(p_bant);
					
				break;
			
			default:
				break;
		}
		
	} else {
		// process packets for this node here
		struct data_packet * p_data;
		p_data = (struct data_packet*)malloc(sizeof(struct data_packet));

		switch ( args->netheader->type ) {
			case PING_REQUEST_PACKET:
				// send back to receiver's address
				Net_sendPong(args->netheader->from);
				break;

			case PING_REPLY_PACKET:
				LOG_LOW( "PONG from %u", args->netheader->from );
				break;

			case USERDEFINED_PACKET+4:
			/*Structure of DATA packet
			
			|###########|-------------|------------------|-------------
			| Netheader | Source_Node | Destination_Node | Data ........
			|###########|-------------|------------------|-------------			
							uint8_t		    uint8_t		   uint8_t[uint8_t]
			*/
				//uint8_t source_node;			
				//uint8_t destination_node;	
				//uint8_t * data
				size_of_data = args->payload_size;
				printf("<<USER_PACKET_1 Received, size = %d\r\n",size_of_data);
				p_data->source_node	  = *(uint8_t *)(args->payload);
				p_data->destination_node = *(uint8_t *)(args->payload)+1;
				p_data->data_point      = (uint8_t *)(args->payload)+2;
				printf("source = %d\r\n",p_data->source_node);
				printf("destination = %d\r\n",p_data->destination_node);
				//printf("data = %s\r\n",data_point);
				if (p_data->destination_node == Configuration.id){
					printf("data from %d is comming.\r\n",p_data->source_node);
					printf("data = %s\r\n",p_data->data_point);
				}else{
					uint8_t next_hop = Find_Next_Hop (p_data); 
					switch (next_hop){
						case 0:
							printf("data routing disappear\r\n");
							break;
						default:
							Send_Data_Packet (p_data, next_hop);
							printf(">>send data packet to next hop %d\r\n",next_hop);
							break;
					}
				}
			break;

			default:
				return false;		// packet not handled (unknown)
		}
		return true;				// packet handled
	}
	return false;					// packet not handled (other destination)
}

void Process_sensorHandler(void* sdata)
{

}

void Process_init()
{
	System_registerCallback( C_RADIO, ( fp_vp )Process_radioHandler );
	System_registerCallback( C_SENSOR, ( fp_vp )Process_sensorHandler );
	
	//Initialize routing table
	for (i = 0; i < SIZE_OF_ROUTING_TABLE; i++){
		routing_table[i].destination_node = 0;
		routing_table[i].next_hop = 0;
		routing_table[i].pheromone_value = 0;
		routing_table[i].flag = 0;
	}
	
	//Initialize fant table
	for(i=0; i < SIZE_OF_FANT_TABLE; i++){
		fant_table[i].source_node = 0;
		fant_table[i].sequence_number = 0;
	}

	//Call the callMe function 5s after system start
	//Timers_add(5*1024, callMe, 0xFFFF);
}

// send data packet with first parameter id second parameter String
COMMAND (SendData, CMDFLAG_SERIAL, cmdargs) {
	
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;
	uint16_t receiver = String_parseUint16(arg, NULL);
	
	
	netpacket_send_args_t npsargs;
	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = receiver;
	npsargs.netheader.type = USERDEFINED_PACKET+4;
	npsargs.netheader.flags = 0;

	arg = ( char* )Cmd_GetNextArg( arg );
	char data_buffer [98];
	sprintf(data_buffer+2,"%s",arg);
	uint8_t   source		= 1;
	uint8_t	  destination   = 4;
	data_buffer[0]			= source;
	data_buffer[1]			= destination;

	npsargs.payload[0].buffer = data_buffer;
	npsargs.payload[0].size = strlen(data_buffer)+1; // include terminating char
	
	Net_send(&npsargs);

}

// Eigene Commands

//Send a FANT packet with specific data
COMMAND(sendFANT, CMDFLAG_SERIAL, cmdargs) {
	struct data_of_fant_bant *p_fant;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	
	char* arg = (char*) cmdargs->args;
	if (count == 0){
		printf("Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter \r\n");
		return;
	}
	
	p_fant->source_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->pre_predecessor = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->destination_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->flag = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->sequence_number = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_fant->hop_counter = atoi(arg);
	
	Send_FANT_BANT(p_fant, PACKET_TYPE_FANT);
	
	return;
}
//Send a BANT packet with specific data
COMMAND(sendBANT, CMDFLAG_SERIAL, cmdargs) {
	struct data_of_fant_bant *p_bant;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	
	char* arg = (char*) cmdargs->args;
	if (count == 0){
		printf("Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter \r\n");
		return;
	}
	
	p_bant->source_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->pre_predecessor = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->destination_node = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->flag = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->sequence_number = atoi(arg);
	arg = ( char* )Cmd_GetNextArg( arg );
	p_bant->hop_counter = atoi(arg);
	
	Send_FANT_BANT(p_bant, PACKET_TYPE_BANT);
	
	return;
}

//Print the Routing table
COMMAND(getRT, CMDFLAG_SERIAL, cmdargs) {
	for (i=0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node == 0)
			return;
			
		printf("Destination: %d ** ", routing_table[i].destination_node);
		printf("Next Hop: %d ** ", routing_table[i].next_hop);
		printf("Pheromone: %d ** ", (uint8_t)routing_table[i].pheromone_value);
		printf("Flag: %d\r\n", routing_table[i].flag);
	}
}

//Print the FANT table
COMMAND(getFT, CMDFLAG_SERIAL, cmdargs) {
	for (i=0; i < SIZE_OF_FANT_TABLE; i++){
		if (fant_table[i].source_node == 0)
			return;
			
		printf("Source of FANT: %d ** ", fant_table[i].source_node);
		printf("Sequence Num: %d \r\n", fant_table[i].sequence_number);
	}
}

//Print the BANT table
COMMAND(getBT, CMDFLAG_SERIAL, cmdargs) {
	for (i=0; i < SIZE_OF_BANT_TABLE; i++){
		if (bant_table[i].source_node == 0)
			return;
			
		printf("Source of BANT: %d ** ", bant_table[i].source_node);
		printf("Sequence Num: %d \r\n", bant_table[i].sequence_number);
	}
}




//Method

// Send (broadcast) a FANT/BANT packet to node id
// @param
// struct data_of_fant_bant *p_fant_bant Pointer to data of fant or fant packet
// return uint8_t: Success/Error-Code
uint8_t Send_FANT_BANT (struct data_of_fant_bant *p_fant_bant, uint8_t packet_type ){

	/*
	|###########|-------------|-----------------|------------------|------|-----------|-------------|
	| Netheader | Source_Node | Pre_Predecessor | Destination_Node | Flag |Seq_Number | Hop_Counter | 
	|###########|-------------|-----------------|------------------|------|-----------|-------------|			
					uint8_t		    uint8_t		      uint8_t		uint8_t  uint16_t	 uint16_t
	*/
	
	uint16_t data_16[4]; 	//array for content of packet
	uint16_t byte_one = 0;	//bytes to concat two uint8_t values
	uint16_t byte_two = 0;
	
	byte_one = p_fant_bant->source_node;
	byte_two = p_fant_bant->pre_predecessor << 8;	//shift to wright place for uint16_t
	data_16[0] = byte_one | byte_two;	//concat source and pre-predecessor
	
	byte_one = p_fant_bant->destination_node;
	byte_two = p_fant_bant->flag << 8;	//shift to wright place for uint16_t
	data_16[1] = byte_one | byte_two;	//concat destination and flag
	data_16[2] = p_fant_bant->sequence_number;			//Sequence number
	data_16[3] = p_fant_bant->hop_counter;			//Hop Counter
	
	netpacket_send_args_t npsargs;

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = 255; 		//Broadcast packet
	if (packet_type == PACKET_TYPE_FANT){//FANT
		npsargs.netheader.type = USERDEFINED_PACKET + PACKET_TYPE_FANT; // FANT
	}
	else if (packet_type == PACKET_TYPE_BANT){//BANT
		npsargs.netheader.type = USERDEFINED_PACKET + PACKET_TYPE_BANT; // BANT
	}
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = data_16;
	npsargs.payload[0].size = sizeof(data_16);
	
	Net_send(&npsargs);
	if (packet_type == PACKET_TYPE_FANT){//FANT
		printf("Send FANT\r\n");
	}
	else if (packet_type == PACKET_TYPE_BANT){//BANT
		printf("Send BANT\r\n");
	}

	return 0;
}

//Check if we have seen this FANT packet already, and add it to the table if not
uint8_t Check_And_Add_FANT_table(struct data_of_fant_bant *p_fant ){
	
	for (i = 0; i < SIZE_OF_FANT_TABLE; i++){
		if (fant_table[i].source_node == p_fant->source_node
			&& fant_table[i].sequence_number == p_fant->sequence_number){
		
			return PACKET_IN_FANT_TABLE;
		}
		if (fant_table[i].source_node == 0){
		
			fant_table[i].source_node = p_fant->source_node;
			fant_table[i].sequence_number = p_fant->sequence_number;
			return PACKET_NOT_IN_FANT_TABLE;
		}
	}
	return 0;
}

//Check if we have seen this BANT packet already, and add it to the table if not
uint8_t Check_And_Add_BANT_table(struct data_of_fant_bant *p_bant ){
	
	for (i = 0; i < SIZE_OF_BANT_TABLE; i++){
		if (bant_table[i].source_node == p_bant->source_node
			&& bant_table[i].sequence_number == p_bant->sequence_number){
		
			return PACKET_IN_BANT_TABLE;
		}
		if (bant_table[i].source_node == 0){
		
			bant_table[i].source_node = p_bant->source_node;
			bant_table[i].sequence_number = p_bant->sequence_number;
			return PACKET_NOT_IN_BANT_TABLE;
		}
	}
	return 0;
}




uint8_t Add_Routing_Table_Entry(uint8_t destination_node, uint8_t next_hop, uint8_t flag, uint16_t hops){
	
	for (i=0; i < SIZE_OF_ROUTING_TABLE; i++){
		if (routing_table[i].destination_node == destination_node
			&& routing_table[i].next_hop == next_hop){
			
			//Entry already in table
			routing_table[i].pheromone_value += 1/hops;
			routing_table[i].flag = flag;
			return ROUTING_TABLE_ENTRY_EXISTS; 
		}
		
		if (routing_table[i].destination_node == 0){
			
			routing_table[i].destination_node = destination_node;
			routing_table[i].next_hop = next_hop;
			routing_table[i].pheromone_value = 1/hops;
			routing_table[i].flag = flag;
			return ROUTING_TABLE_ENTRY_CREATED;
		}
		if (i == SIZE_OF_ROUTING_TABLE-1)
			return ROUTING_TABLE_FULL;
	}
	return 0;
}



































/**
 * @brief	Ping request
 *
 * Send a ping request to another node
 * Example: ping 24
 */
COMMAND( ping, 0, cmdargs )
{
	uint16_t node;

	node = String_parseUint16( cmdargs->args, NULL );
	if ( node > 0 ) {
		Net_sendPing((netaddr_t)node);
	}
}

/**
 * @brief Echo Arguments
 *
 * Testcase 1: long command name
 * Testcase 2: argument splitting
 */
COMMAND( echoargs, CMDFLAG_SERIAL, cmdargs )
{
	uint16_t i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;

	for ( i = 1; i <= count; i++ ) {
		printf("%u: %*s\n", i, strlen(arg), arg);
		arg = ( char* )Cmd_GetNextArg( arg );
	}
	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "%u args found", count );
}

// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);





COMMAND(HelloMSB, CMDFLAG_SERIAL, cmdargs) {
	
	uint16_t n = String_parseUint16(cmdargs->args, NULL);

	while(n--) {
		printf("Hello User called %u times\r\n", counter++);
	}
}

COMMAND(SMyP, CMDFLAG_SERIAL, cmdargs) {
	
	netpacket_send_args_t npsargs;
	uint16_t data = (uint8_t) String_parseUint16(cmdargs->args, NULL);

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = 4; // choose a destination id
	npsargs.netheader.type = USERDEFINED_PACKET+4; // choose a number
	npsargs.netheader.flags = 0;


	npsargs.payload[0].buffer = &data;
	npsargs.payload[0].size = sizeof(data); // include terminating char

	Net_send(&npsargs);

}


COMMAND(genPackets, CMDFLAG_SERIAL, cmdargs) {
	uint16_t i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;
	float interval_1 = 0;
	float interval_2 = 0;
	float interval = 0;
	float tmp_float = 1;
	char * p_point = NULL;
	uint16_t limit = 0;
	uint8_t command = 0; //0 - Stop; 1 - Start

	if (count == 0)
		printf("Please enter some args");
	/*First argument - Start/Stop*/
	if (strstr(arg, "start")){
		command = 1;
		trafficSettings.status = 1;
	}
	else if (strstr(arg, "stop")){
		command = 0;
		trafficSettings.status = 0;
		printf("Sending stopped.\r\n");
		return;
	}
	else {
		printf("Please enter correct parameters [1].\r\n");
		return;
	}

	/*Second argument - Interval*/
	arg = ( char* )Cmd_GetNextArg( arg );
	printf("second arg %s\r\n", arg);
	p_point = strtok(arg, ".");
	if (p_point == NULL)
		printf("Please enter correct parameters [2].\r\n");
	interval_1 = atoi(p_point);
	
	p_point = strtok( NULL, ".");
	if (p_point == NULL)
		interval_2 = 0;
	else{
		arg = ( char* )Cmd_GetNextArg( arg ); //We changed the argument string - jump to next arg
		for (i=0; i<=strlen(p_point);i++){
			tmp_float = 10.0 * tmp_float;
		}
		interval_2 = atoi(p_point);
		printf("atoi(p_point) %d\r\n", atoi(p_point));
		interval_2 = interval_2/tmp_float;
		
	}
	interval = interval_1+interval_2;
	
	/*Third argument - Limit*/
	arg = ( char* )Cmd_GetNextArg( arg );
	printf("third arg %s\r\n", arg);
	limit = atoi(arg);
	printf("limit %d\r\n", limit);



	/*Send the packets*/
	trafficSettings.limit = limit;
	trafficSettings.interval = interval;
	//p_trafficSettings = &trafficSettings;
	printf("trafficSettings.limit %d\r\n", trafficSettings.limit);
	//printf("p_trafficSettings.limit %d\r\n", p_trafficSettings->limit);
	
	Timers_add(256, trafficGenerator, 0xFFFF);
}







void callMe(){

	printf("Blink\r\n");

	do{ P5OUT ^= 0x80; }while(0);
	//Timers_add(2*1024, callMe, 0xFFFF);
}

void blinkBlank(void* loops){

	uint8_t *p_loops = (uint8_t*)loops;
	do{ P5OUT ^= 0x80; }while(0);
	printf("Loop: %d\r\n", *p_loops);
	
	if ((*p_loops)--){
		
		Timers_add(4096, blinkBlank, loops);
	}
}


void trafficGenerator(){
	
	uint16_t data = 5;
	netpacket_send_args_t npsargs;

	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = 5; // choose a destination id
	npsargs.netheader.type = USERDEFINED_PACKET+4; // choose a number
	npsargs.netheader.flags = 0;

	npsargs.payload[0].buffer = &data;
	npsargs.payload[0].size = sizeof(data); // include terminating char
	
	Net_send(&npsargs);
	
	if (trafficSettings.limit-- && trafficSettings.status == 1){
		printf("genPackets limit %d\r\n",trafficSettings.limit);
		//Timers_add(p_trafficSettings->interval*1024, trafficGenerator, (void *) p_trafficSettings);
		Timers_add(2048, trafficGenerator, 0xFFFF);
	}
	
}
