/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <string.h>

uint8_t user_data = 0;
uint8_t * source_point;
uint8_t * destination_point;
uint8_t * data_point;
uint8_t size_of_data = 0;

bool Process_radioHandler(struct netpacket_handler_args* args)
{
	/*
		Notes: * The received packet data becomes invalid when this handler is completed.
	*/

	// Pass packet through command interface
	if ( Cmd_radioHandler( args ) ) {
		// packet handled by messaging module
		return true;
	}

	if ( args->cpaResult == ADRCOMP_OTHER ) {
		// intercept packets for other nodes here
	} else if ( args->cpaResult == ADRCOMP_BROADCAST ) {
		// process broadcast packets here
	} else {
		// process packets for this node here
		switch ( args->netheader->type ) {
			case PING_REQUEST_PACKET:
				// send back to receiver's address
				Net_sendPong(args->netheader->from);
				break;

			case PING_REPLY_PACKET:
				LOG_LOW( "PONG from %u", args->netheader->from );
				break;

			case USERDEFINED_PACKET+1:
				//user_data = *((uint16_t *) (args->payload));
				size_of_data = args->payload_size;
				printf("USER_PACKET_1 Received, size = %d\r\n",size_of_data);
				source_point	  = (uint8_t *)(args->payload);
				destination_point = (uint8_t *)(args->payload)+4;
				data_point      = (uint8_t *)(args->payload)+2;
				printf("source = %d\r\n",*source_point);
				printf("destination = %d\r\n",*destination_point);
				printf("data = %s\r\n",data_point);
			
				//printf("arg = %d\r\n",user_data);
				//Timers_add(1024, callMe, 0xFFFF);
				break;

			default:
				return false;		// packet not handled (unknown)
		}
		return true;				// packet handled
	}
	return false;					// packet not handled (other destination)
}

void Process_sensorHandler(void* sdata)
{

}

void Process_init()
{
	System_registerCallback( C_RADIO, ( fp_vp )Process_radioHandler );
	System_registerCallback( C_SENSOR, ( fp_vp )Process_sensorHandler );
	//Timers_add(5*1024, callMe, 0xFFFF);
}

/**
 * @brief	Ping request
 *
 * Send a ping request to another node
 * Example: ping 24
 */
COMMAND( ping, 0, cmdargs )
{
	uint16_t node;

	node = String_parseUint16( cmdargs->args, NULL );
	if ( node > 0 ) {
		Net_sendPing((netaddr_t)node);
	}
}

/**
 * @brief Echo Arguments
 *
 * Testcase 1: long command name
 * Testcase 2: argument splitting
 */
COMMAND( echoargs, CMDFLAG_SERIAL, cmdargs )
{
	uint16_t i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;

	for ( i = 1; i <= count; i++ ) {
		printf("%u: %*s\n", i, strlen(arg), arg);
		arg = ( char* )Cmd_GetNextArg( arg );
	}
	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "%u args found", count );
}
uint8_t counter = 0;
/**
 * write a new terminal commando
 */
COMMAND (HelloMSB, CMDFLAG_SERIAL, cmdargs) {
	uint16_t n = String_parseUint16(cmdargs->args, NULL);
	while (n--){
		printf("Hello User - called %u times\r\n", counter++);
	}
}

void callMe() {
	P5OUT ^= 0x80;
	Timers_add(1024, callMe, 0xFFFF); // 1024 ticks == second
}

COMMAND (SendP, CMDFLAG_SERIAL, cmdargs) {
	
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;
	uint16_t receiver = String_parseUint16(arg, NULL);
	
	
	netpacket_send_args_t npsargs;
	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = receiver;
	npsargs.netheader.type = USERDEFINED_PACKET+1;
	npsargs.netheader.flags = 0;

	arg = ( char* )Cmd_GetNextArg( arg );
	char data_buffer [98];
	sprintf(data_buffer+2,"%s",arg);
	uint8_t   source		= 1;
	uint8_t	  destination   = 2;
	data_buffer[0]			= source;
	data_buffer[1]			= destination;

//	arg = ( char* )Cmd_GetNextArg( arg );

	//uint16_t data = /*(uint8_t)*/ String_parseUint16(arg, NULL);  // data ist in [0,2^8-1].
	npsargs.payload[0].buffer = data_buffer;
	npsargs.payload[0].size = strlen(data_buffer)+4; // include terminating char
	
	Net_send(&npsargs);

}


/*
COMMAND (SendP, CMDFLAG_SERIAL, cmdargs) {
	
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;
	uint16_t receiver = String_parseUint16(arg, NULL);
	
	
	netpacket_send_args_t npsargs;
	Net_sendArgsInit(&npsargs);

	npsargs.netheader.to = receiver;
	npsargs.netheader.type = USERDEFINED_PACKET+1;
	npsargs.netheader.flags = 0;

	arg = ( char* )Cmd_GetNextArg( arg );
	uint8_t * data = arg;
	char data_buffer [98];
	sprintf(data_buffer,"%s",data);

	npsargs.payload[0].buffer = data;
	npsargs.payload[0].size = strlen(data_buffer)+1; // include terminating char
	
	Net_send(&npsargs);
	printf ("size of data_buffer = %d\r\n",npsargs.payload[0].size);
}
*/
// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);
